﻿using System;
using System.IO;
using System.Reflection;
using System.Text;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Builder;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.DependencyInjection;

using GrapeCity.ActiveReports.Aspnetcore.Viewer;
using GrapeCity.ActiveReports;
using System.Data;
using System.Data.OleDb;

namespace JSViewer_MVCCore
{
    public class Startup
    {
        private static readonly string CurrentDir = Path.GetDirectoryName(Assembly.GetExecutingAssembly().GetName().CodeBase)?.Replace("file:\\", "");
        public static readonly DirectoryInfo ReportsDirectory = new DirectoryInfo(Path.Combine(CurrentDir, "Reports"));

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);
            
            services
                .AddLogging(config =>
                {
                    // Disable the default logging configuration
                    config.ClearProviders();

                    // Enable logging for debug mode only
                    if (Environment.GetEnvironmentVariable("ASPNETCORE_ENVIRONMENT") == Environments.Development)
                    {
                        config.AddConsole();
                    }
                })
                .AddReporting()
                .AddMvc(options => options.EnableEndpointRouting = false);
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            
            app.UseFileServer();

            app.UseReporting(settings =>
            {
                settings.UseCustomStore(GetReport);
                settings.UseCompression = true;
            });

            app.UseMvc();
        }

        public PageReport GetReport(string id)
        {
            PageReport report = new PageReport(new FileInfo(AppContext.BaseDirectory + "\\Reports\\PageReport.rdlx"));
            report.Document.LocateDataSource += Document_LocateDataSource;

            return report;
        }

        private void Document_LocateDataSource(object sender, LocateDataSourceEventArgs args)
        {
            DataSet ds = GetDataSet();
            args.Data = ds.Tables[0]; // Assigning our DataTable to the report.
            // Please Note: The report should contain a DataSource with its provider selected as 'DataSet Provider' and add a DataSet to the DataSource with Fields equal to the Column Name of your DataTable to be able to use these fields in your report.
        }

        private DataSet GetDataSet()
        {
            DataSet ds = new DataSet();
            DataTable dt = new DataTable();

            OleDbConnection con = null;
            con = new OleDbConnection(@"Provider=Microsoft.ACE.OLEDB.12.0;Data Source=" + AppContext.BaseDirectory + @"\Data\NWind.mdb");
            con.Open();
            OleDbDataAdapter da = new OleDbDataAdapter(new OleDbCommand("Select * from Customers", con));
            da.Fill(dt);
            ds.Tables.Add(dt);

            return ds;
        }
    }
}